/*
 * Copyright (C) 2004-2013 L2J DataPack
 * 
 * This file is part of L2J DataPack.
 * 
 * L2J DataPack is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * L2J DataPack is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
package ai.individual;

import ai.npc.AbstractNpcAI;

import com.l2jserver.gameserver.datatables.SpawnTable;
import com.l2jserver.gameserver.model.actor.L2Attackable;
import com.l2jserver.gameserver.model.actor.L2Character;
import com.l2jserver.gameserver.model.actor.L2Npc;
import com.l2jserver.gameserver.model.actor.instance.L2PcInstance;
import com.l2jserver.gameserver.util.Util;

/**
 * Gordon AI
 * @author TOFIZ, malyelfik
 */
public class Gordon extends AbstractNpcAI
{
	private static final int GORDON = 29095;
	
	private Gordon(String name, String descr)
	{
		super(name, descr);
		addSpawnId(GORDON);
		addSeeCreatureId(GORDON);
		addNodeArrivedId(GORDON);
		
if (!com.l2jserver.Config.FIX_onSpawn_for_SpawnTable) {{
		final L2Npc gordon = SpawnTable.getInstance().getFirstSpawn(GORDON).getLastSpawn();
		if (gordon != null)
		{
			onSpawn(gordon);
		}
}}
	}
	
	@Override
	public String onSeeCreature(L2Npc npc, L2Character creature, boolean isSummon)
	{
		if (creature.isPlayer() && ((L2PcInstance) creature).isCursedWeaponEquipped())
		{
			attackPlayer((L2Attackable) npc, (L2PcInstance) creature);
		}
		return super.onSeeCreature(npc, creature, isSummon);
	}
	
	//[JOJO]-------------------------------------------------
	@Override
	public void onNodeArrived(L2Npc npc)
	{
		for (L2PcInstance pc : npc.getKnownList().getKnownPlayers().values()) {
			if (pc.isCursedWeaponEquipped() && Util.checkIfInRange(5000, npc, pc, true)) {
				attackPlayer((L2Attackable) npc, pc);
				break;
			}
		}
	}
	//-------------------------------------------------------
	
	@Override
	public String onSpawn(L2Npc npc)
	{
		((L2Attackable) npc).setCanReturnToSpawnPoint(false);
		return super.onSpawn(npc);
	}
	
	public static void main(String[] args)
	{
		new Gordon(Gordon.class.getSimpleName(), "ai");
	}
}